import warnings

import numpy as np
import pandas as pd
import seaborn as sns
from scipy.io import arff
from sklearn.metrics import classification_report, confusion_matrix
from sklearn.model_selection import train_test_split
from sklearn.neighbors import KNeighborsClassifier
from sklearn.preprocessing import StandardScaler, LabelEncoder

from utilities import ordinal_encode, normalize, plot_confusion_matrix, plot_counts

warnings.filterwarnings("ignore")

# Constants
y_data = 'class'
read_csv = False
csv_path = "nsl-kdd-dataset/" + "fruits_dataset.csv"
arff_path = "nsl-kdd-dataset/" + "KDDTrain+.arff"
sc = StandardScaler()
enc = LabelEncoder()
model_name = "KNN"

if read_csv:
    # Getting data frame (CSV)
    df = pd.read_csv(csv_path)
else:
    # Getting data frame (ARFF)
    data, meta = arff.loadarff(arff_path)
    df = pd.DataFrame(data)
    df = df.applymap(lambda x: x.decode('utf-8') if isinstance(x, bytes) else x)

X = df.select_dtypes(include=[np.number])
y = df[y_data]

is_threat = df[y_data].unique()
if len(is_threat) != 2:
    raise Exception("Target must be a binary decision.")

ordinal_encode(df= df, categories = is_threat, target = y_data)
normalize(df, y_data, sc, enc)

sns.countplot(x = y_data, data = df)
plot_counts(model_name=model_name)

X_train, X_test, y_train, y_test = train_test_split(X, y, test_size = 0.2, random_state = 42)

def train(knn_model):
    knn_model.fit(X_train, y_train)
    y_prediction = knn_model.predict(X_test)
    print("Classification report: \n", classification_report(y_test, y_prediction))
    plot_confusion_matrix(confusion_matrix = confusion_matrix(y_test, y_prediction),
                          accuracy = knn_model.score(X_test, y_test),
                          model_name=model_name)

knn = KNeighborsClassifier()
train(knn)

def predict(prediction_input):
    if len(prediction_input) == 0:
        return
    input_data = pd.DataFrame(prediction_input, columns = X.columns)
    return knn.predict(input_data)
